from hubol.entity import HubolEntity
from owg.util.editor2d.entity import Ephemeral

from owg.util.Calc import fastSin, fastCos
from owg.util.opengl import ObjModel, TextureLib
from owg.util.opengl.EGL import EGL_SHADER_BIT
from world.Global import contentRelative

from bogus.core.BogusCore import egl

from mini.sleuth.game.entities import CellEntity

from hubol.entity import AdvancedDecalBase

state = SimpleNamespace()
state.enter = False
state.carnation = 1
state.prog = 0
entities = []

video_rectangle = world["BobuckReminsceRectangle"]

def get_carnations():
	decals = instance_list(AdvancedDecalBase)
	carnations = filter(lambda decal : decal.texture.get() == "carnation", decals)
	return carnations
	
carnations = get_carnations()

def with_carnations(consume):
	for carnation in carnations:
		consume(carnation)

def free_entities():
	if len(entities) == 0:
		return
	
	for entity in entities:
		entity.destroy()
	del entities[:]

def on(video_tex_entity):
	free_entities()
	entities.append(video_tex_entity)
	entities.append(Cloud(-55, video_tex_entity))
	state.enter = True
	
	with_carnations(lambda entity : entity.setVisible(True))
	alarm_restartable(animate_carnations).start(30)

def horizontal_flip(decal):
	sign = (decal.subimage.get() * 2 - 1) * state.carnation
	
	xscale = abs(decal.getXScale()) * sign
	yscale = decal.getYScale()
	decal.setScale(xscale, yscale, False)
	
def animate_carnations(alarm):
	state.carnation *= -1
	with_carnations(horizontal_flip)

	if state.enter:
		alarm.restart(30)
	
def off():
	state.enter = False
	with_carnations(lambda entity : entity.setVisible(False))
	
def step():
	if state.enter and state.prog < 1:
		state.prog += 0.06
	elif not state.enter and state.prog > 0:
		state.prog -= 0.1
	
	if state.prog <= 0:
		free_entities()
		
def start():
	with_carnations(lambda entity : entity.setVisible(False))
	with_carnations(horizontal_flip)
	
class Cloud(HubolEntity, Ephemeral):
	def __init__(self, depth, video_tex_entity):
		HubolEntity.__init__(self, list, depth)
		self.video_tex_entity = video_tex_entity
		self.steps = 0
		self.prog_prev = 0

	def render(self):
		if state.prog <= 0 or self.video_tex_entity.isDestroyed():
			return
			
		tex = self.video_tex_entity.videoTex.tex()
		
		prog = CellEntity.lerp(self.prog_prev, state.prog, game.getInterp())
		steps = CellEntity.lerp(self.steps - 1, self.steps, game.getInterp())
		
		target_scale_x = video_rectangle.getXScale()
		target_scale_y = video_rectangle.getYScale()		
		
		scale = prog
		xscale = target_scale_x * scale
		yscale = target_scale_y * scale
	
		egl.modelviewMatrix().push()
		egl.modelviewMatrix().translated(video_rectangle.getX() + target_scale_x/2, video_rectangle.getY() + target_scale_y/2, 0)
		egl.modelviewMatrix().scaled(xscale, yscale, 1)
		egl.modelviewMatrix().translatef(-0.5, -0.5, 0)
		
		egl.pushAttribs(EGL_SHADER_BIT)
		egl.useProgram(game.alphaMask)
		egl.enableTexture2D(tex, 0)
		egl.enableTexture2D(1, TextureLib.tex("soft quad mono"), 0)
		egl.nwSquare.render()
		egl.disableTexture(1)
		egl.popAttribs()
		
		egl.modelviewMatrix().pop()
		
	def beginStep(self):
		self.prog_prev = state.prog
		
	def step(self):
		self.steps += 1

