from owg.util.euclidian import V2D
from audio import Audio

def slide_to_bogus(seconds_to_slide):
	begin_slide(bogus_side_pos, seconds_to_slide)
	
def slide_to_meatfiend(seconds_to_slide):
	begin_slide(meatfiend_side_pos, seconds_to_slide)

def bite(bite_amount):
	state.sandwich_amount -= bite_amount
	if state.sandwich_amount < 0:
		state.sandwich_amount = 0
	for scaleable_decal in scaleable_decals:
		scaleable_decal.scale(state.sandwich_amount)
	bite_clip.play()
	
	return has_bites_left()
	
def has_bites_left():
	return state.sandwich_amount > 0.001
	
def step():
	if state.delta_pos == 0:
		return
	working_vec.set(state.current_pos)
	
	state.current_pos.accelerateTowards(state.target_pos, state.delta_pos)
	working_vec.sub(state.current_pos)
	diff_x = -working_vec.x()
	diff_y = -working_vec.y()
	
	for decal in scaleable_decals:
		decal.state.pos.add(diff_x, diff_y)
		
	if state.current_pos.equals(state.target_pos):
		state.delta_pos = 0
		slide_clip.stop()
			
def begin_slide(target_pos, seconds_to_slide):
	state.target_pos = target_pos
	state.delta_pos = state.current_pos.clone().sub(target_pos).len() / 30 / seconds_to_slide
	slide_clip.play()

class ScaleableDecal:
	def __init__(self, decal):
		self.state = decal.state
		self.initial_scale = decal.state.scale.clone()
		
	def scale(self, factor):
		self.state.scale.set(self.initial_scale).multiply(factor)

scaleable_decals = map(ScaleableDecal, world["MeatFiendSandwichGroup"])

def v2d(placeable):
	return V2D(placeable.getX(), placeable.getY())

meatfiend_side_pos = v2d(world["MeatFiendSandwichGroup"])
bogus_side_pos = v2d(world["SandwichDestination"])

working_vec = V2D()

bite_clip = Audio.getClip("bite1")
bite_clip.setPosition(None, 0.8)

slide_clip = Audio.getClip("drag")
slide_clip.setLooping(True)

state = SimpleNamespace()

state.current_pos = meatfiend_side_pos.clone()
state.target_pos = state.current_pos.clone()

state.delta_pos = 0

state.sandwich_amount = 1.0
		
		
		